@@ -5,6 +5,7 @@
 const Legacy = imports._legacy;
 const {Gio, GjsPrivate, GLib, GObject} = imports.gi;
 const {_registerType} = imports._common;
+const Gi = imports._gi;
 
 let Gtk;
 let BuilderScope;
@@ -35,14 +36,17 @@ function _init() {
 
     Gtk.Widget.prototype._init = function (params) {
         let wrapper = this;
+        const klass = this.constructor;
 
-        if (wrapper.constructor[Gtk.template]) {
+        if (klass[Gtk.template]) {
             if (!BuilderScope) {
-                Gtk.Widget.set_connect_func.call(wrapper.constructor,
+                Gtk.Widget.set_connect_func.call(klass,
                     (builder, obj, signalName, handlerName, connectObj, flags) => {
+                        const objects = builder.get_objects();
+                        const thisObj = objects.find(o => o instanceof klass);
                         const swapped = flags & GObject.ConnectFlags.SWAPPED;
                         const closure = _createClosure(
-                            builder, wrapper, handlerName, swapped, connectObj);
+                            builder, thisObj, handlerName, swapped, connectObj);
 
                         if (flags & GObject.ConnectFlags.AFTER)
                             obj.connect_after(signalName, closure);
@@ -54,17 +58,17 @@ function _init() {
 
         wrapper = GObject.Object.prototype._init.call(wrapper, params) ?? wrapper;
 
-        if (wrapper.constructor[Gtk.template]) {
-            let children = wrapper.constructor[Gtk.children] || [];
+        if (klass[Gtk.template]) {
+            let children = klass[Gtk.children] ?? [];
             for (let child of children) {
                 wrapper[child.replace(/-/g, '_')] =
-                    wrapper.get_template_child(wrapper.constructor, child);
+                    wrapper.get_template_child(klass, child);
             }
 
-            let internalChildren = wrapper.constructor[Gtk.internalChildren] || [];
+            let internalChildren = klass[Gtk.internalChildren] ?? [];
             for (let child of internalChildren) {
                 wrapper[`_${child.replace(/-/g, '_')}`] =
-                    wrapper.get_template_child(wrapper.constructor, child);
+                    wrapper.get_template_child(klass, child);
             }
         }
 
@@ -156,16 +160,18 @@ function _init() {
         }, class extends GObject.Object {
             vfunc_create_closure(builder, handlerName, flags, connectObject) {
                 const swapped = flags & Gtk.BuilderClosureFlags.SWAPPED;
-                return _createClosure(
-                    builder, builder.get_current_object(),
-                    handlerName, swapped, connectObject);
+                const thisArg = builder.get_current_object();
+                return Gi.associateClosure(
+                    connectObject ?? thisArg,
+                    _createClosure(builder, thisArg, handlerName, swapped, connectObject)
+                );
             }
         });
     }
 }
 
 function _createClosure(builder, thisArg, handlerName, swapped, connectObject) {
-    connectObject = connectObject || thisArg;
+    connectObject = connectObject ?? thisArg;
 
     if (swapped) {
         throw new Error('Unsupported template signal flag "swapped"');
